@@ -1,6 +1,7 @@
 from pyalign.tests import TestCase
+from typing import Iterator
 
-import pyalign.problem
+import pyalign.problems
 import pyalign.solve
 import pyalign.gaps
 
@@ -10,14 +11,13 @@ class TestGotoh(TestCase):
 		# the following problem is taken from the slides
 		# about Gotoh by Rolf Backofen.
 
-		pf = pyalign.problem.ProblemFactory(
-			pyalign.problem.Binary(eq=0, ne=1),
+		pf = pyalign.problems.general(
+			pyalign.problems.Equality(eq=0, ne=1),
 			direction="minimize")
 
 		solver = pyalign.solve.GlobalSolver(
 			gap_cost=pyalign.gaps.AffineGapCost(4, 1),
-			direction="minimize",
-			generate="alignment[all, optimal]")
+			codomain=Iterator[pyalign.solve.Alignment])
 
 		problem = pf.new_problem("CC", "ACCT")
 		alignments = solver.solve(problem)
